/* 
    Ming-Ching Tiew mctiew@yahoo.com
    9th March 2005
    GPL Source
    g++ lzmac.cpp compress3.cpp -LPath -llzma -o lzmac
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

ZEXTERN int ZEXPORT compress3 OF((Bytef *dest,   uLongf *destLen,
                                  const Bytef *source, uLong sourceLen, 
                                  int lc, int lp, int pb, 
                                  int level));
                                  
int main(int argc, char** argv)
{
   FILE * fp;
   int prop0;
   unsigned long zero = 0;
   unsigned int dictsize;
   unsigned long size;
   unsigned long outsize;
   unsigned long decsize;
   unsigned long c_byte = 0;
   int result;
   int lc, lp, pb;
   Byte *s, *d;
   if ( argc == 1 || argc > 3 )
   {
      fprintf( stderr, "usage: lzmac sourcefile [props]\n"
      	   "eg lzmac input_file     > output_file\n"
      	   "   lzmac input_file 302 > output_file\n"
      	   "   lzmac input_file 500 > output_file\n");
      	   exit(1); 
   }   	   
   if ( argc == 2 )
   {
      lc = 3; lp = 0; pb = 2;
   }   
   if ( argc == 3 )
   {
      if ( strlen(argv[2]) != 3 )   
      {
         lc = 3; lp = 0; pb = 2;
      }
      else 
      {
         lc = argv[2][0] - '0'; 
         lp = argv[2][1] - '0'; 
         pb = argv[2][2] - '0'; 
      }
   }
   // sanitize values
   if ( lc < 0 || lc > 8) lc = 3;
   if ( lp < 0 || lp > 4) lp = 0;
   if ( pb < 0 || pb > 4) pb = 2;
   //    
   fp = fopen( argv[1], "r" );
   if( fp == NULL )
   {
   	fprintf(stderr, "%s: argument one has to be an existing file!\n", argv[0] );
   	exit(1);
   }
   fseek(fp, 0L, SEEK_END);
   size = ftell(fp);
   s = new Byte [ size];
   outsize =  size ;
   d  = new Byte [ outsize ];
   // move back up   
   fseek(fp, 0L, SEEK_SET);
   // read the whole file
   fread(s, size, 1, fp); 
   fclose( fp);
   // 
   result = compress3( d, &outsize, s, size,  lc, lp, pb, 9 );
   if( result != 0 )
   {
   	fprintf(stderr, "%s: compression has failed with error code %d!\n", argv[0], result);
   	exit(2);
   }
   //  
   if ( outsize > size )
   {
   	fprintf( stderr, "%s: output bigger than input!\n", argv[0]);
   	exit(3);
   }
   // write the headers
   prop0 = pb * 9 * 5 + lp * 9 + lc; 
   dictsize = (1<<23);
   fwrite( (char*)&prop0, 1, 1, stdout);    // write properties
   fwrite( (char*)&dictsize, 4, 1, stdout); //write dictionary size
   fwrite( (char*)&size, 4, 1, stdout);     //decompressed size
   fwrite( (char*)&zero, 4, 1, stdout);     //decompressed size
   fwrite( d, outsize, 1, stdout);
   //  
   delete s;
   delete d;
   return 0;
}
